#!/usr/bin/env python

# k3watch by Leigh WA5ZNU Leigh@wa5znu.org

# References:
# Code and ideas from
# http://marcin.af.gliwice.pl/if-then-else-20070121143245
# http://www.pygtk.org/pygtk2reference/class-gtkstatusicon.html
# http://www.devshed.com/c/a/Python/Basic-Threading-in-Python/1/
# http://www.thescripts.com/forum/thread717463.html

import gobject
import gtk
import pygtk
import threading
import time
import pynotify
import os
import sys
import urllib2
from lxml import etree
from k3rig import K3Rig, FirmwareLevels
from weblib import WebResource

ONE_SECOND = 1
ONE_MINUTE = 60*1
TEN_MINUTES = 60*10
ONE_HOUR = 60*60
FIRMWARE_URI='http://wa5znu.org/2008/02/k3watch/bin/list-elecraft-firmware.cgi'
mainThread=None

class MainThread(threading.Thread):
    stopthread = threading.Event()

    def __init__(self):
        threading.Thread.__init__(self)
        self.notifier = K3FirmwareStatusIcon()
        self.rig = RigThread(K3Rig())
        self.site = FirmwareLister()
        self.lastMessage = None
        self.waitTime = ONE_MINUTE # check for update every minute until we get a result,  then move to every ten minutes
        self.rig.notifier = self.notifier
        self.site.notifier = self.notifier
        self.notifier.rig = self.rig
        self.notifier.site = self.site

    def start(self):
        self.rig.start()
        threading.Thread.start(self)
       
    def run(self):
        time.sleep(1)                   # some race condition with notifier causes offscreen notification panels
        while not self.stopthread.isSet():
            if self.notifier.get_visible():
                site_levels = self.site.firmware_levels()
                if site_levels:
                    if self.rig.firmware_levels():
                        self.waitTime = ONE_HOUR
                        if site_levels.isOlder(self.rig.firmware_levels()):
                            self.notifyUser(site_levels)
                time.sleep(self.waitTime)

    def notifyUser(self, l):
        message = "New: %s\nRig: %s" % (l.version_string(), self.rig.version_string())
        if not self.alreadyNotified(message):
            self.notifier.show_notification("New K3 Firmware Available", message)
            self.lastMessage = message

    def alreadyNotified(self, message):
        return self.lastMessage == message

    def stop(self):
        print "mainThread stop"
        self.rig.stop()
        self.stopthread.set()
        gtk.main_quit()


class FirmwareLister:
    def __init__(self):
        self.levels = None
        self.etag = None
        self.message = None

    def version_string(self):
        if self.levels:
            return self.levels.version_string()
        else:
            return "(Unable to read firmware site)"

    def firmware_levels(self):
        try:
            resource = WebResource()
            (s,etag) = resource.get(FIRMWARE_URI, self.levels, self.etag)
            if etag:
                self.etag = etag
            if (s != self.levels):
                if s.tag == "versions":
                    self.levels = FirmwareLevels()
                    self.levels.mcu = s.findtext("mcu")
                    self.levels.dsp1 = s.findtext("dsp1")
                    self.levels.dsp2 = s.findtext("dsp2")
                    return self.levels
                if s.tag == "error":
                    self.notifier.show_notification("Error from firmware site: %s", s.findtext("message"))
                    return None
                if s.tag == "warning":
                    self.notifier.show_notification("Warning from firmware site: %s", s.findtext("message"))
                    return None
                else:
                    self.notifier.show_notification("Unknown response from firmware site: %s", s.tag)
                    return None
            else:
                return self.levels
        except Exception, e:
            self.notifier.show_notification("Error from firmware site", str(e))
            return None


class RigThread(threading.Thread):
    stopthread = threading.Event()

    def __init__(self, k3):
        threading.Thread.__init__(self)
        self.k3 = k3
        self.wasBad = False
        self.wasGood = False
        self.notifier = None
        self.max_quick_tries = 10       # try ten times to open before going into long waits
       
    def run(self):
        tries_since_good = 0
        while not self.stopthread.isSet():
            if self.k3.update():
                tries_since_good = 0
                if (self.wasBad):
                    # self.notifier.show_notification("Info", "Read K3 Firmware Revisions after a hiatus")
                    self.wasBad = False
                time.sleep(TEN_MINUTES)
            else:
                if not self.wasGood:
                    self.wasBad = True
                tries_since_good += 1
                if tries_since_good < self.max_quick_tries:
                    time.sleep(ONE_SECOND)
                else:
                    time.sleep(ONE_MINUTE)
        sys.exit(0)

    def firmware_levels(self):
        self.k3.update()
        return self.k3.firmware_levels()

    def version_string(self):
        l = self.k3.firmware_levels()
        if l:
            return l.version_string()
        else:
            return "(Unable to read K3 status)"

    def stop(self):
        print "rig stop"
        self.stopthread.set()

    
class K3FirmwareStatusIcon(gtk.StatusIcon):
    aboutMessage = "K3 Firmware Watch -- checks for new K3 firmware.\n\nRig: %s\nSite: %s\n\n%s"

    def __init__(self):
        try:
            gtk.gdk.threads_enter()
            gtk.StatusIcon.__init__(self)
            self.rig = None
            self.site = None
            self.set_from_file("logo.png")
            self.set_tooltip("K3 Firmware Watch")
            self.n=None

            menu = gtk.Menu()
            menuItem = gtk.ImageMenuItem(gtk.STOCK_ABOUT)
            menuItem.connect('activate', self.activate_icon_cb)
            menu.append(menuItem)
            menuItem = gtk.ImageMenuItem(gtk.STOCK_QUIT)
            menuItem.connect('activate', self.quit_cb, self)
            menu.append(menuItem)

            self.connect('activate', self.activate_icon_cb)
            self.connect('popup-menu', self.popup_menu_cb, menu)

            self.set_visible(True)
        finally:
            gtk.gdk.threads_leave()

    def quit_cb(self, widget, data = None):
        if data:
            data.set_visible(False)
        mainThread.stop()

    def popup_menu_cb(self, widget, button, time, data = None):
        if button == 3:
            if data:
                data.show_all()
                data.popup(None, None, None, 3, time)
        pass

    def activate_icon_cb(self, widget, data = None):
        rig_levels = self.rig.firmware_levels()
        site_levels = self.site.firmware_levels()
        updateMsg = ""
        if site_levels.isOlder(rig_levels):
            updateMsg = "Update needed"
        message = self.aboutMessage % (rig_levels.version_string(), site_levels.version_string(), updateMsg)
        msg_box = gtk.MessageDialog(parent = None,
                                   buttons = gtk.BUTTONS_OK,
                                   message_format = message)
        msg_box.run()
        msg_box.destroy()

    def show_notification(self, title, message):
        self.title = title
        self.message = message
        gobject.timeout_add(0, self._do_show_notification)

    def _do_show_notification(self):
        n = pynotify.Notification(self.title, self.message)

    # This doesn't work until Fedora Core 8.  It exits python.
    # Also there's no K3 Firmware Updater for Linux yet anyway
        #        # http://www.nabble.com/Fedora-8-Update%3A-notify-python-0.1.1-2.fc8-td15241097.html#a15241097
        #        def on_notification_activated(n, action):
        #            try:
        #                print "Received event: NotificationActivated(%s, %s)" % (id, action)
        #                activate_updater()
        #            except Exception, ex:
        #                print ex
        #
        #        def activate_updater(self):
        #            print "Starting firmware updater. Please hold..."
        #            os.system(K3_FIRMWARE_PROGRAM)
        #            gtk.main_quit()
        #
        #        n.add_action('Start Update', 'Start Update', on_notification_activated)
        n.attach_to_status_icon(self)
        n.show()
        self.n=n
        return False

def main():
    pynotify.init("K3 Firmware Watch")
    gobject.threads_init()
    global mainThread
    mainThread = MainThread()
    mainThread.start()
    return gtk.main()

if __name__ == "__main__":
    sys.exit(main())
