#!/usr/bin/python
# Elecraft K3 Rig Control Python Utilities
# Copyright (C) 2006, 2007, 2008 Leigh L. Klotz, Jr <Leigh@WA5ZNU.org>
# Licensed under Academic Free License 3.0 (See LICENSE)

# Put this in a cron job and run every six hours.
# TODO: Cache this to a file and check the date on the file; don't fetch more than every 6 hours.

from ftplib import FTP
from sha import sha 
import time
import os

ftp_site="ftp.elecraft.com"
ftp_dir="/K3/firmware"
cache_file="/home/wa5znu/2008/02/k3watch/data/firmware.xml"

def readVersions():
    try:
        cache_mtime = os.path.getmtime(cache_file)
        if (time.time() - cache_mtime) > 60*60*6:
            return writeCache(parseSite(readSite()))+"\n<!-- new -->"
        else:
            return readCache()
    except Exception, ex:
        return writeCache(parseSite(readSite()))+"\n<!-- err -->"

def writeCache(data):
    f = open(cache_file, 'w')
    f.write(data)
    f.close()
    return data

def readCache():
    return open(cache_file, 'r').read()+"\n<!-- cached -->"

def readSite():
    ftp = FTP(ftp_site)
    ftp.login()
    ftp.cwd(ftp_dir)
    data = ftp.nlst()
    return data

def parseSite(data):
    sb="<?xml version='1.0'?>\n"
    sb = sb + "<versions>\n"
    for item in data:
        item = item.lower()
        if (item.endswith(".hex")):
            velement = parseVersion(item)
            if velement:
                sb = sb + " " + velement + "\n"
    sb = sb + "</versions>"
    return sb

def parseVersion(item):
    if (item.startswith("haux")):
        return "<dsp2>%s</dsp2>" % (item[4:-4])
    if (item.startswith("hdsp")):
        return "<dsp1>%s</dsp1>" % (item[4:-4])
    if (item.startswith("hmcu")):
        return "<mcu>%s</mcu>" % (item[4:-4])
    if (item.startswith("dtbl")):
        return "<dtbl>%s</dtbl>" % (item[4:-4])
    if (item.startswith("tboot")):
        return "<tboot>%s</tboot>" % (item[5:-4])

def etag(v):
#    file_version = os.stat(sys.argv[0]).st_mtime
    etag = sha(v)
#    etag.update(str(file_version))
    return '"%s"' % etag.hexdigest()

v = readVersions()
ee = etag(v)

if_none_match = os.environ.get('HTTP_IF_NONE_MATCH', '')
if if_none_match and ee == if_none_match:
    print "Status: 304 Not Modified"
    print ""
else:
    print "Status: 200 OK"
    print "Content-Type: application/xml"
    print "ETag: %s" % (ee)
    print "Content-Length: %d" % (len(v))
    print ""
    print v


